<?php
/**
 * Created by JetBrains PhpStorm.
 * User: jorgefonseca
 * Date: 16/08/13
 * Time: 21:12
 * To change this template use File | Settings | File Templates.
 */

namespace Importacao\Logic\Integracao;


use Importacao\Logic\Integracao\Schema\Tabela;

class GeradorSQL {

    const DROP             = 'DROP TABLE';
    const CREATE           = 'CREATE TABLE';
    const INSERT           = 'INSERT INTO ';
    const FORMATO_DATA     = 'ALTER SESSION SET NLS_DATE_FORMAT = ';
    const FORMATO_NUMERICO = 'ALTER SESSION SET NLS_NUMERIC_CHARACTERS = ';
    const DEFINE_OFF       = 'SET DEFINE OFF;';
    const COMMIT           = 'COMMIT;';

    /** @var  $tabela Tabela */
    private $tabela;
    private $formatoData = '';
    private $formatoNumero = '';

    public function __construct($formatoData = 'DD/MM/RR', $formatoNumero = ', ') {
        $this->setFormatoData($formatoData);
        $this->setFormatoNumero($formatoNumero);
    }

    /**
     * @return string
     */
    public function getFormatoData()
    {
        return $this->formatoData;
    }

    /**
     * @return string
     */
    public function getFormatoNumero()
    {
        return $this->formatoNumero;
    }

    /**
     * @return mixed
     */
    public function getTabela()
    {
        return $this->tabela;
    }

    /**
     * @param \Migracao\Logic\Integracao\Schema\Tabela $tabela
     */
    public function setTabela(Tabela $tabela)
    {
        $this->tabela = $tabela;
    }

    public function geraCabecalho() {
        $sql = self::DEFINE_OFF.PHP_EOL;
        $sql .= $this->getFormatoNumero().PHP_EOL;
        $sql .= $this->getFormatoData().PHP_EOL.PHP_EOL;
        return $sql;
    }

    public function geraDrop() {
        $sql = self::DROP." {$this->tabela->getNome()};".PHP_EOL;
        return $sql;
    }

    public function geraCreate() {
        $sql  = '';
        $sql .= self::CREATE." \"{$this->tabela->getNome()}\" (";
        $sql .= $this->geraColunas();
        return $sql;
    }

    private function geraColunas() {
        $sql = PHP_EOL."\t";
        $sql .= $this->geraColunasPadrao();
        $colunas = $this->tabela->getColunas();
        $count = count($colunas) - 1;
        foreach($colunas as $key => $coluna) {
            if ($coluna->getTamanhoMaximo()) {
                $tipo = "{$coluna->getTipo()}({$coluna->getTamanhoMaximo()})";
            } else {
                $tipo = $coluna->getTipo();
            }
            if ($key != $count) {
                $sql .= "\"{$coluna->getNome()}\" $tipo," . PHP_EOL . "\t";
            } else {
                $sql .= "\"{$coluna->getNome()}\" $tipo" . PHP_EOL . ');' . PHP_EOL;
            }
        }
        return $sql;
    }

    private function geraColunasPadrao() {
        $nrorg     = "\"NRORG\" NUMBER,".PHP_EOL."\t";
        //$nrempresa = "\"NREMPRESA\" NUMBER,".PHP_EOL."\t";
        return $nrorg;
    }

    public function geraInsert($autoCommit = true) {
        $colunas = $this->tabela->getColunas();
        $sql = '';
        $count = count($this->tabela->getColunas()) - 1;
        foreach($this->tabela->getRows() as $chave => $row) {
            $sqlValor = '';
            foreach ($row as $key => $value) {
                $existeAspasSimples = (strpos($value, '\'') !== false) ? true : false;
                if ($existeAspasSimples) {
                    $sqlValor .= "q'[$value]', ";
                } else {
                    if ($value === 'sysdate')
                        $sqlValor .= $value.', ';
                    else
                        $sqlValor .= "'{$value}', ";
                }
            }
            $sqlValor = substr($sqlValor, 0, count($sqlValor) - 3);

            $sql .= self::INSERT."\"{$this->tabela->getNome()}\" (";
            $count2 = count($colunas) - 1;
            for ($i = 0; $i < count($colunas); $i++) {
                if ($i != $count2) {
                    $sql .= $colunas[$i]->getNome().", ";
                } else {
                    $sql .= $colunas[$i]->getNome();
                }
            }
            $sql .= ") VALUES ($sqlValor);".PHP_EOL;
            if ($chave != 0 && ($chave % 200) == 0) {
                $sql .= self::COMMIT.PHP_EOL;
            }
        }

        if ($autoCommit) {
            $sql .= self::COMMIT.PHP_EOL;
        }

        return $sql;
    }

    public function setFormatoData($formato) {
        if ($formato) {
            $this->formatoData = self::FORMATO_DATA . "'$formato';";
        }
    }

    public function setFormatoNumero($formato) {
        if ($formato) {
            $this->formatoNumero = self::FORMATO_NUMERICO . "'$formato';";
        }
    }

}